@@ -1,13 +1,13 @@
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/init.d/halt.sh,v 1.40 2003/10/14 00:02:21 azarah Exp $
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/init.d/halt.sh,v 1.44 2003/11/10 00:29:47 azarah Exp $
 
 
 # We need to properly terminate devfsd to save the permissions
 if [ -n "`ps --no-heading -C 'devfsd'`" ]
 then
 	ebegin "Stopping devfsd"
-	killall -15 devfsd &> /dev/null
+	killall -15 devfsd &>/dev/null
 	eend $?
 elif [ ! -e /dev/.devfsd -a -e /dev/.udev ]
 then
@@ -18,28 +18,20 @@ then
 	eend 0
 fi
 
-ebegin "Sending all processes the TERM signal"
-killall5 -15 &> /dev/null
-eend $?
-sleep 5
-ebegin "Sending all processes the KILL signal"
-killall5 -9 &> /dev/null
-eend $?
-
 # Try to unmount all tmpfs filesystems not in use, else a deadlock may
 # occure, bug #13599.
-umount -at tmpfs &> /dev/null
+umount -at tmpfs &>/dev/null
 
-if [ -n "`swapon -s 2> /dev/null`" ]
+if [ -n "`swapon -s 2>/dev/null`" ]
 then
 	ebegin "Deactivating swap"
-	swapoff -a &> /dev/null
+	swapoff -a &>/dev/null
 	eend $?
 fi
 
 # Write a reboot record to /var/log/wtmp before unmounting
 
-halt -w &> /dev/null
+halt -w &>/dev/null
 
 # Unmounting should use /proc/mounts and work with/without devfsd running
 
@@ -62,18 +54,18 @@ remaining="`awk '!/^#/ && $1 ~ /^\/dev\/loop/ && $2 != "/" {print $2}' /proc/mou
 		if [ "${retry}" -lt 3 ]
 		then
 			ebegin "Unmounting loopback filesystems (retry)"
-			umount -d ${remaining} &> /dev/null
+			umount -d ${remaining} &>/dev/null
 			eend $? "Failed to unmount filesystems this retry"
 		else
 			ebegin "Unmounting loopback filesystems"
-			umount -d ${remaining} &> /dev/null
+			umount -d ${remaining} &>/dev/null
 			eend $? "Failed to unmount filesystems"
 		fi
 		
 		remaining="`awk '!/^#/ && $1 ~ /^\/dev\/loop/ && $2 != "/" {print $2}' /proc/mounts | sort -r`"
 		[ -z "${remaining}" ] && break
 		
-		/bin/fuser -k -m ${sig} ${remaining} &> /dev/null
+		/bin/fuser -k -m ${sig} ${remaining} &>/dev/null
 		sleep 5
 		retry=$((${retry} - 1))
 		sig=-9
@@ -100,9 +92,16 @@ do
 	
 	if [ "${do_unmount}" = "yes" ] && \
 	   [ "${x}" != "/" -a "${x}" != "/dev" -a "${x}" != "/proc" -a \
-	     "${x}" != "/sys" ]
+	     "${x}" != "/sys" -a "${x}" != "/mnt/livecd" ]
 	then
-		umount -f -r ${x} &> /dev/null
+		umount -f ${x} &>/dev/null || {
+		
+			# Kill processes still using this mount
+			/bin/fuser -k -m -9 "${x}" &>/dev/null
+			sleep 2
+			# Now try to unmount it again ...
+			umount -f -r ${x} &>/dev/null
+		}
 	fi
 done
 eend 0
@@ -112,7 +111,7 @@ if [ -x /sbin/vgchange ] && [ -f /etc/lvmtab -o -d /etc/lvm ] && \
    [ -d /proc/lvm  -o "`grep device-mapper /proc/misc 2>/dev/null`" ]
 then
 	ebegin "Shutting down the Logical Volume Manager"
-	/sbin/vgchange -a n > /dev/null
+	/sbin/vgchange -a n >/dev/null
 	eend $? "Failed to shut LVM down"
 fi
 
@@ -129,24 +128,29 @@ ups_kill_power() {
 	fi
 }
 
+mount_readonly() {
+	local x=
+	local retval=0
+	
+	for x in `awk '{ print $2 }' /proc/mounts`
+	do
+		mount -n -f -o remount,ro ${x} &>/dev/null
+		retval=$((${retval} + $?))
+	done
+
+	return ${retval}
+}
+
 ebegin "Remounting remaining filesystems readonly"
 # Get better results with a sync and sleep
-sync;sync
+sync; sync
 sleep 1
-sync
-sleep 1
-if [ -n "${CDBOOT}" ]
+if ! mount_readonly
 then
-	#LiveCD: don't unmount the read-only livecd loopback filesystem or all our commands will disappear
-	mount | cut -f1 -d" " | grep -v livecd | xargs umount -r -n -t  nodevfs,noproc,nosysfs,notmpfs &>/dev/null
-else
-	umount -a -r -n -t nodevfs,noproc,nosysfs,notmpfs &>/dev/null
-fi
-if [ "$?" -ne 0 ]
-then
-	killall5 -9  &> /dev/null
-	umount -a -r -n -l -d -f -t nodevfs,noproc,nosysfs &> /dev/null
-	if [ "$?" -ne 0 ]
+	killall5 -9  &>/dev/null
+	sync; sync
+	sleep 1
+	if ! mount_readonly
 	then
 		eend 1
 		sync; sync
@@ -172,4 +176,5 @@ fi
 
 [ -f /etc/killpower ] && ups_kill_power
 
+
 # vim:ts=4
