@@ -1,22 +1,10 @@
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.44 2003/10/14 19:03:15 azarah Exp $
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.47 2003/10/26 17:48:23 azarah Exp $
 
 
-set -a
-
 RC_GOT_FUNCTIONS="yes"
 
-umask 022
-
-if [ -z "${EBUILD}" ]
-then
-	# Setup a basic $PATH.  Just add system default to existing.
-	# This should solve both /sbin and /usr/sbin not present when
-	# doing 'su -c foo', or for something like:  PATH= rcscript start
-	PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/sbin:${PATH}"
-fi
-
 # daemontools dir
 SVCDIR="/var/lib/supervise"
 
@@ -54,77 +42,6 @@ RC_USE_CONFIG_PROFILE="yes"
 [ -f /etc/conf.d/rc ] && source /etc/conf.d/rc
 
 
-getcols() {
-	echo "$2"
-}
-
-# Should we use colors ?
-if [ -n "${EBUILD}" ] && [ "${*/depend}" = "$*" ]
-then
-	# Check user pref in portage
-	RC_NOCOLOR="$(python -c 'import portage; print portage.settings["NOCOLOR"]' 2> /dev/null)"
-
-	[ "${RC_NOCOLOR}" = "true" ] && RC_NOCOLOR="yes"
-
-elif [ -n "${EBUILD}" ] && [ "${*/depend}" != "$*" ]
-then
-	# We do not want colors or stty to run during emerge depend
-	RC_NOCOLOR="yes"
-	
-elif [ "$(/sbin/consoletype 2> /dev/null)" = "serial" ]
-then
-	# We do not want colors on serial terminals
-	RC_NOCOLOR="yes"
-else
-	# Lastly check if the user disabled it with --nocolor argument
-	for arg in $*
-	do
-		case "${arg}" in
-			--nocolor)
-				RC_NOCOLOR="yes"
-				;;
-		esac
-	done
-fi
-
-if [ "${RC_NOCOLOR}" = "yes" ]
-then
-	COLS="25 80"
-	ENDCOL=
-
-	if [ -n "${EBUILD}" ] && [ "${*/depend}" = "$*" ]
-	then
-		stty cols 80 &>/dev/null
-		stty rows 25 &>/dev/null
-	fi
-else
-	COLS="`stty size 2> /dev/null`"
-	COLS="`getcols ${COLS}`"
-	COLS=$((${COLS} - 7))
-	ENDCOL=$'\e[A\e['${COLS}'G'    # Now, ${ENDCOL} will move us to the end of the
-	                               # column;  irregardless of character width
-fi
-
-if [ "${RC_NOCOLOR}" = "yes" ]
-then
-	GOOD=
-	WARN=
-	BAD=
-	NORMAL=
-
-	HILITE=
-	BRACKET=
-else
-	GOOD=$'\e[32;01m'
-	WARN=$'\e[33;01m'
-	BAD=$'\e[31;01m'
-	NORMAL=$'\e[0m'
-
-	HILITE=$'\e[36;01m'
-	BRACKET=$'\e[34;01m'
-fi
-
-
 # void get_bootconfig()
 #
 #    Get the BOOTLEVEL and SOFTLEVEL by setting
@@ -206,7 +123,11 @@ splash_init() {
 	pb_scripts=0
 	pb_rate=0
 
-	[ ! -e /proc/splash -o ! -e /sbin/splash ] && return 0
+	if [ ! -x /sbin/splash ] || \
+	   [ -e /proc/version -a ! -e /proc/splash ]
+	then
+		return 0
+	fi
 	
 	if [ -f /etc/conf.d/bootsplash.conf ]
 	then
@@ -226,37 +147,38 @@ splash_init() {
 
 	export pb_init pb_count pb_scripts pb_rate
 }
+
 #
-# void splash_calc (char *argv1, char *svcdir)
+# void splash_calc (void)
 #
 splash_calc() {
-	pb_runs=(/etc/runlevels/$1/*)
-	pb_runb=(/etc/runlevels/boot/*)
+	pb_runs=($(dolisting "/etc/runlevels/${SOFTLEVEL}/"))
+	pb_runb=($(dolisting "/etc/runlevels/${BOOTLEVEL}/"))
 	pb_scripts=${#pb_runs[*]}
 	pb_boot=${#pb_runb[*]}
 
-	[ ! -e /proc/splash -o ! -e /sbin/splash ] && return 0
+	[ ! -e /proc/splash -o ! -x /sbin/splash ] && return 0
 
-    if [ -f /etc/conf.d/bootsplash.conf ]
+	if [ -f /etc/conf.d/bootsplash.conf ]
 	then
-        . /etc/conf.d/bootsplash.conf
+		. /etc/conf.d/bootsplash.conf
 
 		if [ -n "${PROGRESS_SYSINIT_RATE}" ]
 		then
 			init_rate=$((65535*${PROGRESS_SYSINIT_RATE}/100))
 		fi
 		
-        if [ -n "${PROGRESS_BOOT_RATE}" ]
+		if [ -n "${PROGRESS_BOOT_RATE}" ]
 		then
-            boot_rate=$((65535*${PROGRESS_BOOT_RATE}/100))
-        fi
+			boot_rate=$((65535*${PROGRESS_BOOT_RATE}/100))
+		fi
 	fi
 
 	# In runlevel boot we have 5 already started scripts
 	#
-	if [ "${RUNLEVEL}" = "S" -a "${1}" = "boot" ]
+	if [ "${RUNLEVEL}" = "S" -a "${SOFTLEVEL}" = "boot" ]
 	then
-		pb_started=($(dolisting "${2}/started/"))
+		pb_started=($(dolisting "${svcdir}/started/"))
 		pb_scripts=$((${pb_boot} - ${#pb_started[*]}))
 		pb_init=16383
 		pb_rate=26213
@@ -266,9 +188,9 @@ splash_calc() {
 			pb_init="${init_rate}"
 			pb_rate=$((${init_rate} + ${boot_rate}))
 		fi
-	elif [ "${RUNLEVEL}" = "0" -o "${RUNLEVEL}" = "6" ]
+	elif [ "${SOFTLEVEL}" = "reboot" -o "${SOFTLEVEL}" = "shutdown" ]
 	then
-		pb_started=($(dolisting "${2}/started/"))
+		pb_started=($(dolisting "${svcdir}/started/"))
 		pb_scripts=${#pb_started[*]}
 		pb_rate=65534
 	else
@@ -280,82 +202,124 @@ splash_calc() {
 		fi
 	fi
 	
-	echo "pb_init=${pb_init}" > "${2}/progress"
-	echo "pb_rate=${pb_rate}" >> "${2}/progress"
-	echo "pb_count=${pb_count}" >> "${2}/progress"
-	echo "pb_scripts=${pb_scripts}" >> "${2}/progress"
+	echo "pb_init=${pb_init}" > "${svcdir}/progress"
+	echo "pb_rate=${pb_rate}" >> "${svcdir}/progress"
+	echo "pb_count=${pb_count}" >> "${svcdir}/progress"
+	echo "pb_scripts=${pb_scripts}" >> "${svcdir}/progress"
 }
+
 #
-# void splash_update (char *svcdir, char *myscript, char *action)
+# void splash_update (char *fsstate, char *myscript, char *action)
 #
 splash_update() {
-	[ ! -e /proc/splash -o ! -e /sbin/splash ] && return 0
+	local fsstate="$1"
+	local myscript="$2"
+	local action="$3"
 	
-	if [ "${1}" = "inline" ]
+	[ ! -e /proc/splash -o ! -x /sbin/splash ] && return 0
+	
+	if [ "${fsstate}" = "inline" ]
 	then
-		/sbin/splash "$2" "$3"
+		/sbin/splash "${myscript}" "${action}"
 		pb_count=$((${pb_count} + 1))
 
 		# Only needed for splash_debug() 
-		pb_execed="${pb_execed} ${2:-inline}"
+		pb_execed="${pb_execed} ${myscript:-inline}"
 	else
 		# Update only runlevel scripts, no dependancies (only true for startup)
-		if [ -z "$(ls -1 \"${1}/softscripts/\" 2> /dev/null | grep \"${2}\")" ]
+		if [ ! -L "${svcdir}/softscripts/${myscript}" ]
 		then
-			[ "${RUNLEVEL}" != "0" -a "${RUNLEVEL}" != "6" ] && return
+			[ "${SOFTLEVEL}" != "reboot" -a \
+			  "${SOFTLEVEL}" != "shutdown" ] && return
 		fi
 		# Source the current progress bar state
-		test -f "${1}/progress" && source "${1}/progress"
+		[ -f "${svcdir}/progress" ] && source "${svcdir}/progress"
 
 		# Do not update an already executed script
 		for x in ${pb_execed}
 		do
-			[ "${x}" = "${2}" ] && return
+			[ "${x}" = "${myscript}" ] && return
 		done	
 		
-		/sbin/splash "$2" "$3"
+		/sbin/splash "${myscript}" "${action}"
 		pb_count=$((${pb_count} + 1))
 		
-		echo "pb_init=${pb_init}" > "${1}/progress"
-		echo "pb_rate=${pb_rate}" >> "${1}/progress"
-		echo "pb_count=${pb_count}" >> "${1}/progress"
-		echo "pb_scripts=${pb_scripts}" >> "${1}/progress"
-		echo "pb_execed=\"${pb_execed} ${2}\"" >> "${1}/progress"
+		echo "pb_init=${pb_init}" > "${svcdir}/progress"
+		echo "pb_rate=${pb_rate}" >> "${svcdir}/progress"
+		echo "pb_count=${pb_count}" >> "${svcdir}/progress"
+		echo "pb_scripts=${pb_scripts}" >> "${svcdir}/progress"
+		echo "pb_execed=\"${pb_execed} ${myscript}\"" >> "${svcdir}/progress"
 	fi
 }
+
 #
-# void splash_debug (char *argv1)
+# void splash_debug (char *softlevel)
 #
 splash_debug() {
+	local softlevel="$1"
+
+	[ ! -e /proc/splash -o ! -x /sbin/splash ] && return 0
+	
 	if [ -f /etc/conf.d/bootsplash.conf ]
 	then
 		source /etc/conf.d/bootsplash.conf
 
-		[ "${BOOTSPLASH_DEBUG}" = "yes" -a -n "${1}" ] || return
+		[ "${BOOTSPLASH_DEBUG}" = "yes" -a -n "${softlevel}" ] || return
 		
 		if [ -f "${svcdir}/progress" ]
 		then
-			cat "${svcdir}/progress" > "/var/log/bootsplash.${1}"
+			cat "${svcdir}/progress" > "/var/log/bootsplash.${softlevel}"
 		else
-			echo "pb_init=${pb_init}" > "/var/log/bootsplash.${1}"
-			echo "pb_rate=${pb_rate}" >> "/var/log/bootsplash.${1}"
-			echo "pb_count=${pb_count}" >> "/var/log/bootsplash.${1}"
-			echo "pb_scripts=${pb_scripts}" >> "/var/log/bootsplash.${1}"
-			echo "pb_execed=\"${pb_execed}\"" >> "/var/log/bootsplash.${1}"
+			echo "pb_init=${pb_init}" > "/var/log/bootsplash.${softlevel}"
+			echo "pb_rate=${pb_rate}" >> "/var/log/bootsplash.${softlevel}"
+			echo "pb_count=${pb_count}" >> "/var/log/bootsplash.${softlevel}"
+			echo "pb_scripts=${pb_scripts}" >> "/var/log/bootsplash.${softlevel}"
+			echo "pb_execed=\"${pb_execed}\"" >> "/var/log/bootsplash.${softlevel}"
 		fi	
 	fi
 }
 
-if [ -e /proc/splash -a -e /sbin/splash ]
-then
-	rc_splash() {
-		/sbin/splash $*
-	}
-else
-	rc_splash() {
-		return 0
-	}
-fi
+update_splash_wrappers() {
+	if [ -x /sbin/splash ] && \
+	   ([ ! -e /proc/version ] || \
+	    [ -e /proc/version -a -e /proc/splash ])
+	then
+		rc_splash() {
+			/sbin/splash $*
+		}
+		rc_splash_init() {
+			splash_init $*
+		}
+		rc_splash_calc() {
+			splash_calc $*
+		}
+		rc_splash_update() {
+			splash_update $*
+		}
+		rc_splash_debug() {
+			splash_debug $*
+		}
+	else
+		rc_splash() {
+			return 0
+		}
+		rc_splash_init() {
+			return 0
+		}
+		rc_splash_calc() {
+			return 0
+		}
+		rc_splash_update() {
+			return 0
+		}
+		rc_splash_debug() {
+			return 0
+		}
+	fi
+
+	export rc_splash rc_splash_init rc_splash_calc \
+		rc_splash_update rc_splash_debug
+}
 
 # void esyslog(char* priority, char* tag, char* message)
 #
@@ -772,14 +736,83 @@ add_suffix() {
 	return 0
 }
 
-set +a
+getcols() {
+	echo "$2"
+}
 
 if [ -z "${EBUILD}" ]
 then
+	# Setup a basic $PATH.  Just add system default to existing.
+	# This should solve both /sbin and /usr/sbin not present when
+	# doing 'su -c foo', or for something like:  PATH= rcscript start
+	PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/sbin:${PATH}"
+
+	if [ "$(/sbin/consoletype 2> /dev/null)" = "serial" ]
+	then
+		# We do not want colors on serial terminals
+		RC_NOCOLOR="yes"
+	fi
+	
+	for arg in $*
+	do
+		case "${arg}" in
+			# Lastly check if the user disabled it with --nocolor argument
+			--nocolor)
+				RC_NOCOLOR="yes"
+				;;
+		esac
+	done
+
 	if [ -e "/proc/cmdline" ]
 	then
 		setup_defaultlevels
 	fi
+
+	update_splash_wrappers
+else
+	# Should we use colors ?
+	if [ "${*/depend}" = "$*" ]
+	then
+		# Check user pref in portage
+		RC_NOCOLOR="$(portageq envvar NOCOLOR 2>/dev/null)"
+		
+		[ "${RC_NOCOLOR}" = "true" ] && RC_NOCOLOR="yes"
+	else
+		# We do not want colors or stty to run during emerge depend
+		RC_NOCOLOR="yes"
+	fi                                                                                                                       
+fi
+
+if [ "${RC_NOCOLOR}" = "yes" ]
+then
+	COLS="25 80"
+	ENDCOL=
+	
+	GOOD=
+	WARN=
+	BAD=
+	NORMAL=
+	HILITE=
+	BRACKET=
+	
+	if [ -n "${EBUILD}" ] && [ "${*/depend}" = "$*" ]
+	then
+		stty cols 80 &>/dev/null
+		stty rows 25 &>/dev/null
+	fi
+else
+	COLS="`stty size 2> /dev/null`"
+	COLS="`getcols ${COLS}`"
+	COLS=$((${COLS} - 7))
+	ENDCOL=$'\e[A\e['${COLS}'G'    # Now, ${ENDCOL} will move us to the end of the
+	                               # column;  irregardless of character width
+	
+	GOOD=$'\e[32;01m'
+	WARN=$'\e[33;01m'
+	BAD=$'\e[31;01m'
+	NORMAL=$'\e[0m'
+	HILITE=$'\e[36;01m'
+	BRACKET=$'\e[34;01m'
 fi
 
 
