@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.26 2003/10/14 19:03:15 azarah Exp $
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.29 2003/10/26 17:48:23 azarah Exp $
 
 
 # Common functions
@@ -198,7 +198,7 @@ svc_stop() {
 		eerror "        \"${myservice}\" is still up."
 	else
 		# Now that deps are stopped, stop our service
-		(filter_environ; stop)
+		stop
 		retval="$?"
 	fi
 	
@@ -315,7 +315,7 @@ svc_start() {
 			retval=1
 		elif [ "${retval}" -eq 0 ] && ! broken "${myservice}" &>/dev/null
 		then
-			(filter_environ; start)
+			start
 			retval="$?"
 		fi
 
@@ -409,7 +409,7 @@ svc_homegrown() {
 			if typeset -F "${x}" &>/dev/null
 			then
 				# Run the homegrown function
-				(filter_environ; "${x}")
+				"${x}"
 				
 				return $?
 			fi
@@ -447,11 +447,11 @@ do
 	case "${arg}" in
 	stop)
 		svc_stop
-		splash_update "${svcdir}" "$myservice" "stop"
+		rc_splash_update "live" "${myservice}" "stop"
 		;;
 	start)
 		svc_start
-		splash_update "${svcdir}" "$myservice" "start"
+		rc_splash_update "live" "${myservice}" "start"
 		;;
 	needsme|ineed|usesme|iuse|broken)
 		list_depend_trace "${arg}"
